# ---------------------------------------
#   程序：timezone.py
#   版本：0.1
#   作者：lds
#   日期：2022-05-18
#   语言：Python 3.X
#   说明：常用的函数集合
# ---------------------------------------

from django.utils.timezone import localtime, now


def get_local_now(format_string='%Y-%m-%d %H:%M:%S'):
    """
    获取本地的当前时间
    """
    return localtime(now()).strftime(format_string)



# ---------------------------------------
#   程序：admin.py
#   版本：0.5
#   作者：lds
#   日期：2021-11-05
#   语言：Python 3.X
#   说明：django 后台管理相关的动作和功能
# ---------------------------------------

import csv

from ilds.file import human_size

from django.contrib import admin
from django.contrib import messages
from urllib.parse import quote as urlquote

from django.http import HttpResponse
from django.db.models import Sum

from djlds.model import ModelFields


# https://docs.djangoproject.com/zh-hans/4.2/ref/contrib/admin/actions/


class ExportCsvMixin:
    """
    导出 CSV 格式文件的动作

    支持自定义的内容：
    # 设置导出 Csv 文件的编码
    csv_charset = 'utf-8-sig'  # gb2312 有些字符不支持
    # 自定义导出 ForeignKey 字段内容
    related_fields = {'ForeignKey字段': {'fields': ['字段1', '字段2'], }}
    # 排除导出字段
    csv_export_exclude = []
    # 导出 csv 文件的名字
    export_csv_name = '后台数据'

    # 添加到动作
    actions = ['export_as_csv', 'export_all_as_csv', ]
    """

    # def get_actions(self, request):
    #     # 这个可以用来直接添加到动作中，但是有两个问题
    #     # 1 这个要在类继承的最前面
    #     # 2 self.export_as_csv 需要三个参数 modeladmin, request, queryset
    #     actions = super().get_actions(request)
    #     # print(actions)
    #     # 添加自定义动作到动作列表
    #     actions['copy_data_to_target_database'] = (
    #         self.export_as_csv,
    #         'copy_data_to_target_database',
    #         '导出所选的 %(verbose_name_plural)s 为csv文件',
    #     )
    #
    #     return actions

    def export_as_csv(self, request, queryset, is_all=False):
        if getattr(self, 'using', None):
            queryset = queryset.using(self.using)

        meta = self.model._meta
        related_fields = getattr(self, 'related_fields', {})
        export_exclude = getattr(self, 'csv_export_exclude', [])

        # 获取需要导出的 ForeignKey 字段
        for related_field, data in related_fields.items():
            if getattr(meta.concrete_model, related_field, None):
                cg_model = getattr(meta.concrete_model, related_field).field.related_model
                # 另外一种获取模型的方法
                # [field for field in meta.fields if field.name == 'cg_hetong_bianhao'][0].remote_field.model
                m = ModelFields(cg_model)
            else:
                raise ValueError(f'没有找到 {related_field} 模型')
            data['names'] = []
            for field in data['fields']:
                verbose = m.field_to_verbose(field)
                if verbose is None:
                    raise ValueError(f'没有找到 {field} 字段')
                # 写入标题
                data['names'].append(verbose)

        field_names = []
        # 写入标题
        field_verbose_name = []

        for field in meta.fields:
            if field.name in export_exclude or field.verbose_name in export_exclude:
                continue
            field_names.append(field.name)
            if field.name in related_fields:
                field_verbose_name.extend(related_fields[field.name]['names'])
            else:
                field_verbose_name.append(field.verbose_name)

        response = HttpResponse(content_type='text/csv', charset=getattr(self, 'csv_charset', 'utf-8-sig'))
        export_csv_name = getattr(self, 'export_csv_name', meta)
        if is_all:
            export_filename = urlquote(f'导出 {export_csv_name} 全部内容')
        else:
            export_filename = urlquote(f'导出 {export_csv_name} 中选择的内容')
        response['Content-Disposition'] = f"attachment; filename={export_filename}.csv"
        writer = csv.writer(response)

        writer.writerow(field_verbose_name)
        for obj in queryset:
            rows = []
            for field in field_names:
                if field in related_fields:
                    related_obj = getattr(obj, field, None)
                    if related_obj:
                        for related_field in related_fields[field]['fields']:
                            val = getattr(related_obj, related_field)
                            rows.append(val)
                    else:
                        rows.extend(['' for _ in related_fields[field]['fields']])
                else:
                    val = getattr(obj, field)
                    rows.append(val)
            writer.writerow(rows)

        return response

    def export_all_as_csv(self, request, queryset):
        return self.export_as_csv(request, self.model.objects.all(), is_all=True)

    # Export Selected
    export_as_csv.short_description = "导出已选中（csv）"
    export_all_as_csv.short_description = "导出全部（csv）"


class MultiDBModelAdmin(admin.ModelAdmin):
    """
    在 Django 管理界面中使用多数据库
    https://docs.djangoproject.com/zh-hans/3.0/topics/db/multi-db/#exposing-multiple-databases-in-django-s-admin-interface
    """
    # 要使用的数据库
    using = None

    def save_model(self, request, obj, form, change):
        """
        告诉 Django 将对象保存到“指定”数据库
        """
        obj.save(using=self.using)

    def delete_model(self, request, obj):
        """
        告诉 Django 从“指定”数据库中删除对象
        """
        obj.delete(using=self.using)

    def get_queryset(self, request):
        """
        告诉 Django 在“指定”数据库中查找对象
        """
        return super(MultiDBModelAdmin, self).get_queryset(request).using(self.using)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        """
        告诉 Django 使用“指定”数据库上的查询填充 ForeignKey 小部件
        """
        return super(MultiDBModelAdmin, self).formfield_for_foreignkey(db_field, request, using=self.using, **kwargs)

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        """
        告诉 Django 使用“指定”数据库上的查询填充 ManyToMany 小部件
        """
        return super(MultiDBModelAdmin, self).formfield_for_manytomany(db_field, request, using=self.using, **kwargs)


class BaseUserAdmin(admin.ModelAdmin):
    """
    1. 用来处理文件管理、文件分类、这些 model 的 用户字段自动补充
    2. 用来针对 queryset 过滤当前用户的数据
    """
    # 在编辑、新增页面上排除
    exclude = ('write_uid', 'create_uid',)

    def get_queryset(self, request):
        """
        使当前登录的用户只能看到自己内容
        """
        qs = super(BaseUserAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs
        # 过滤当前用户只能看到自己的文章
        return qs.filter(owner=request.user)

    def save_model(self, request, obj, form, change):
        if change:
            obj.write_uid = request.user
        else:
            # obj.write_uid = request.user
            obj.create_uid = request.user
        return super(BaseUserAdmin, self).save_model(request, obj, form, change)


class SeparateActions:
    """
    分割动作，方便区分不同功能
    """

    actions = ['separate_1', 'separate_2', 'separate_3', 'separate_4', 'separate_5', 'separate_actions']

    def separate_1(self, request, queryset):
        self.message_user(request, '未选择动作', level=messages.WARNING)

    def separate_2(self, request, queryset):
        self.message_user(request, '未选择动作', level=messages.WARNING)

    def separate_3(self, request, queryset):
        self.message_user(request, '未选择动作', level=messages.WARNING)

    def separate_4(self, request, queryset):
        self.message_user(request, '未选择动作', level=messages.WARNING)

    def separate_5(self, request, queryset):
        self.message_user(request, '未选择动作', level=messages.WARNING)

    def separate_actions(self, request, queryset):
        self.message_user(request, '未选择动作', level=messages.WARNING)

    separate_1.short_description = "---------"
    separate_2.short_description = "---------"
    separate_3.short_description = "---------"
    separate_4.short_description = "---------"
    separate_5.short_description = "---------"
    separate_actions.short_description = "---------"


class FavoriteActions:
    # actions = ['add_favorite', 'del_favorite']

    def add_favorite(self, request, queryset):
        """
        添加到收藏
        """
        if request.user.is_superuser:
            try:
                count = queryset.filter(is_favorite=False).count()
                update = queryset.update(is_favorite=True)
                self.message_user(request, f'收藏: {count}:{update}')
            except Exception as e:
                self.message_user(request, f'修改失败: {e}', level=messages.ERROR)
        else:
            self.message_user(request, '修改失败，没有权限!', level=messages.ERROR)

    def del_favorite(self, request, queryset):
        """
        取消收藏
        """
        if request.user.is_superuser:
            try:
                count = queryset.filter(is_favorite=True).count()
                update = queryset.update(is_favorite=False)
                self.message_user(request, f'取消收藏: {count}:{update}')
            except Exception as e:
                self.message_user(request, f'修改失败: {e}', level=messages.ERROR)
        else:
            self.message_user(request, '修改失败，没有权限!', level=messages.ERROR)

    add_favorite.short_description = "收藏"
    del_favorite.short_description = "取消收藏"


class UpdateQuerysetAction:
    """
    自定义更新类型的动作
    """

    # actions = ['自定义更新类型的动作']

    def _update_queryset_action(self, name, request, queryset, is_superuser=False, **kwargs):
        """
        更新查询集
        """
        if is_superuser and not request.user.is_superuser:
            self.message_user(request, f'管理员才有权限{name}!', level=messages.ERROR)
            return
        try:
            self.message_user(request, f'{name} {queryset.update(**kwargs)}')
        except Exception as e:
            self.message_user(request, f'{name}失败: {e}', level=messages.ERROR)

    # _update_queryset_action.short_description = "自定义更新类型的动作"


class CopyQuerysetAction:
    # actions = ['copy_queryset']

    def copy_queryset(self, request, queryset):
        """
        复制选中的内容(只支持复制单个内容)
        """
        count = queryset.count()
        if count > 1:
            self.message_user(request, f'不能复制 {count} 个内容，我们只支持复制单个内容', level=messages.ERROR)
        else:
            obj = queryset[0]
            obj.id = None
            obj.save()
            self.message_user(request, f'复制 {obj} 成功', level=messages.INFO)

    copy_queryset.short_description = "复制选中的内容(只支持复制单个内容)"


class FileSizeQuerysetAction:
    # actions = ['action_calc_file_size']

    def action_calc_file_size(self, request, queryset):
        """
        计算选择内容的文件大小
        """
        field = getattr(self, 'file_size_field', 'file_size')
        exclude_count = queryset.filter(**{field: None}).count()
        if exclude_count:
            info = f'（跳过 {exclude_count} 个文件大小为空的内容）'
            queryset = queryset.exclude(**{field: None})
        else:
            info = ''
        file_size = queryset.aggregate(**{field: Sum(field)})[field]
        # print(human_size(file_size))
        self.message_user(request, f'选择内容的文件大小：{human_size(file_size)}' + info, level=messages.INFO)

    action_calc_file_size.short_description = "计算选择内容的文件大小"


class QuickDeleteQuerysetAction:
    # actions = ['action_quick_delete']

    def action_quick_delete(self, request, queryset):
        """
        直接删除选择的内容（不需要确认，谨慎操作）
        """

        self.message_user(request, f'{queryset.delete()}', level=messages.INFO)

    action_quick_delete.short_description = "直接删除选择的内容（不需要确认，谨慎操作）"



version = '1.3.9'

# 因为日期格式问题，所以下面的版本号不能使用
# 主版本号 (Major Version): 较大的功能变化或结构调整
# 次版本号 (Minor Version): 较小的功能变化或一些新功能的添加
# 修订版本号 (Patch Version): bug 修复或非常小的变化
# 构建号 (Build Number): 1构建号 （Build Number）： 小的修复或改进，不会引入新的功能



import operator
import os
from functools import reduce
# from collections import OrderedDict

from colorama import Fore, Back, Style

# from django.forms.models import model_to_dict  # 获取模型实例的字典
from django.db.models.base import ModelBase, Model
from django.conf import settings

from django.apps import apps
from django.db import models
# group_by 这个分组 我喜欢
from django.db.models import Aggregate, Avg, Count, Max, Min, StdDev, Sum, Variance
from django.db.models import QuerySet
# # from django.db.models import FileField
from django.db.models import Sum
from django.forms.models import model_to_dict

from djlds.xlsx_util import xl_col_to_name

"""
20181203 添加 搜索多个字段的函数
20181203 ModelFields 添加 获取模型对象
20181203 添加了获取模型的字段，名字和类型的类
20181130 添加 确认是否修改线上数据库（本地操作，因为如果没有修改会直接退出，防止误操作）
20170814 整理了下模型调用
20170814 把模型操作分离出来，以后调用模型都通过这里
20180807 去掉不通用的模型操作，并改成共享库

批量导入例子
loadList = []
kwargs = 需要添加的字段字典
loadList.append(需要添加的库模型(**kwargs))
print('成功导入 %s 行' % len(models_ku.objects.bulk_create(loadList)))
"""


def get_model(value, model_dict=None, is_exact=True) -> Model:
    """
    通过库的名字或模型的名字获取模型

    例子：
        ku_model = get_model('酷狗', model_dict=None, is_exact=True)
        if not ku_model:
            raise RuntimeError('没有找到模型！')

    :param value: 要获取的模型，如果本身就是模型，直接返回
    :param model_dict: 从模型字典（模型名字：模型）中获取模型
    :param is_exact: 精确模式，控制是否匹配文件名中包含模型名的情况
    :return: Model
    """

    if isinstance(value, ModelBase):
        return value

    _model_dict = {app.__name__: app for app in apps.get_models() if
                   'django.contrib.' not in str(app) and '.models.' in str(app)}
    # print(_model_dict)

    # 检查模型的名称
    if value in _model_dict:
        return _model_dict[value]
    elif model_dict is not None:
        if is_exact:
            # 如果是精确模式，匹配以后就可以返回
            if value in model_dict:
                return model_dict[value]
        else:
            # 不是精确模式，主要是处理文件名中包含模型名字的情况
            for ku_name in model_dict.keys():
                # print(ku_name)
                if ku_name in value:
                    return model_dict[ku_name]


def get_search_results(queryset, search_fields, search_term):
    """
    搜索，返回包含查询集的
    来自：from django.contrib.admin.options import ModelAdmin
    """

    # 应用关键字搜索。
    def construct_search(field_name):
        if field_name.startswith('^'):
            return "%s__istartswith" % field_name[1:]
        elif field_name.startswith('='):
            return "%s__iexact" % field_name[1:]
        elif field_name.startswith('@'):
            return "%s__search" % field_name[1:]
        else:
            return "%s__icontains" % field_name

    if search_fields and search_term:
        orm_lookups = [construct_search(str(search_field))
                       for search_field in search_fields]
        for bit in search_term.split():
            or_queries = [models.Q(**{orm_lookup: bit})
                          for orm_lookup in orm_lookups]
            #
            queryset = queryset.filter(reduce(operator.or_, or_queries))

    return queryset


class ModelFields:
    """
    获取模型的字段，名字和类型。

    例子：
    from djlds.model import ModelFields

    print(ModelFields(mymodel).verbose_to_type('标题'))
    for i in ModelFields(mymodel).iter('type'):
        print(i)
    """

    def __init__(self, model, exclude=None):
        """
        初始化
        """
        self.model = model
        self.field_list = []
        self.verbose_list = []
        self.type_list = []
        self.fields = []

        if exclude is None:
            exclude = []
        for f in model._meta.fields:  # model._meta._get_fields(reverse=False) 包括 ManyToMany 字段
            field_name = f.name
            verbose_name = str(f.verbose_name)
            type_name = type(f).__name__
            if field_name in exclude or verbose_name in exclude or type_name in exclude:
                continue
            self.field_list.append(field_name)
            self.verbose_list.append(verbose_name)
            self.type_list.append(type_name)
            self.fields.append(f)

        self.count = len(self.field_list)
        # print(self.field_list)
        # print(self.verbose_list)
        # print(self.type_list)

    def field_to_verbose(self, field):
        """
        通过 field_name 获取 verbose_name
        """
        if field in self.field_list:
            return self.verbose_list[self.field_list.index(field)]

    def field_to_type(self, field):
        """
        通过 field_name 获取 type_name
        """
        if field in self.field_list:
            return self.type_list[self.field_list.index(field)]

    def field_dict(self):
        """
        获取键为 field，值为 verbose_name 的字典
        :
        """
        return {field: self.field_to_verbose(field) for field in self.field_list}

    def verbose_to_field(self, field):
        """
        通过 verbose_name 获取 field_name
        """
        if field in self.verbose_list:
            return self.field_list[self.verbose_list.index(field)]

    def verbose_to_type(self, field):
        """
        通过 verbose_name 获取 type_name
        """
        if field in self.verbose_list:
            return self.type_list[self.verbose_list.index(field)]

    def verbose_dict(self):
        """
        获取键为 verbose_name，值为 field 的字典
        :
        """
        return {verbose: self.verbose_to_field(verbose) for verbose in self.verbose_list}

    def get_field(self, name):
        """
        通过 名字获取 field 属性
        按 field_name -> verbose_name 的顺序查找，
        已经找到就不在继续查找，没有找到返回空
        """
        if name in self.field_list:
            return self.fields[self.field_list.index(name)]
        elif name in self.verbose_list:
            return self.fields[self.verbose_list.index(name)]

    def iter(self, *args):
        """
        获取指定字段的生成器
        """
        iter_dict = {
            'field': self.field_list,
            'verbose': self.verbose_list,
            'type': self.type_list,
        }
        if not args:
            args = ['field']
        else:
            if [arg for arg in args if arg not in iter_dict]:
                raise ValueError(f"参数错误：{args}，可用参数为：'field', 'verbose', 'type'")

        # print([iter_dict[arg][1] for arg in args])
        # print([i for i in range(self.count)])
        return ([iter_dict[arg][i] for arg in args] for i in range(self.count))


class TableData(ModelFields):
    """
    转换 Excel 数据为 Django 模型数据

    我们继承自 ModelFields，所以可以使用其所有功能

    # 例子
    table = TableData(models)
    alias = {'替换内容1':'替换为1', '替换内容2':'替换为2'}
    cannot_import = table.set_title(headers, **alias)
    print(table.exclude_info)
    if r:
        cannot_import_dict = {d['name']: '' for d in cannot_import}
        print(f"不能导入的字段：{cannot_import_dict}")
        raise ValueError(f'字段没有导入：{r}')
    # print(r, table.table_fields, table.index_list)
    """

    def __init__(self, model, exclude=None):
        super().__init__(model, exclude)
        # 标题
        self.title = None
        # Excel列数据对应数据库字段的索引
        self.index_list = None
        # Excel列数据对应数据库的字段名
        self.table_fields = None
        self.duplicate_info = {}
        self.exclude_fields = []
        self.additional_fields = []
        self.additional_field_name = None  # 存储不在模型中的附加字段，一般会使用 additional_data，为 None 的时候不保存附加内容

    def set_title(self, row, exclude=None, is_skip_blank=True, **kwargs):
        """
        设置标题行内容

        通过标题行对应的列索引获取数据

        :param row: 标题行内容
        :param exclude: 要排除的内容
        :param is_skip_blank: 是否跳过空白标题
        :param kwargs: 用来修正数据，数据库的表述和列表标题不同的时候使用
        :return: 不能导入的字段
        """
        if not isinstance(row, list):
            raise ValueError('Excel行必须是列表类型！')
        if exclude is None:
            exclude = []

        index_list = []
        table_fields = []
        cannot_import = []
        self.exclude_fields = []
        self.duplicate_info = {}
        self.additional_fields = []
        for i, name in enumerate(row):
            if name in exclude:
                self.exclude_fields.append(name)
                if self.additional_field_name:
                    self.additional_fields.append(name)
                continue
            # 跳过空白标题
            if is_skip_blank and not name.strip():
                continue
            # 优先使用别名
            if name in kwargs:
                name = kwargs[name]
            field = self.verbose_to_field(name)
            if field:
                # print(field)
                if name in exclude:
                    self.exclude_fields.append(name)
                    if self.additional_field_name:
                        self.additional_fields.append(name)
                else:
                    index_list.append(i)
                    table_fields.append(field)
            elif self.additional_field_name:
                self.additional_fields.append(name)
            else:
                cannot_import.append({'column': xl_col_to_name(i), 'name': name})

        self.title = row
        self.index_list = index_list
        self.table_fields = table_fields
        self.count = len(table_fields)

        # 检查是否有重复字段
        if self.count != len(set(self.table_fields)):
            check_duplicates = {}
            for i, field in enumerate(self.table_fields):
                if field in check_duplicates:
                    if field not in self.duplicate_info:
                        self.duplicate_info[field] = [{'column': xl_col_to_name(check_duplicates[field]),
                                                       'name': self.title[check_duplicates[field]]}]
                    self.duplicate_info[field].append(
                        {'column': xl_col_to_name(self.index_list[i]), 'name': self.title[self.index_list[i]]})
                else:
                    check_duplicates[field] = self.index_list[i]

        return cannot_import

    def row_to_dict(self, row):
        """
        获取 Django 模型使用的字典数据

        :param row: Excel行
        :return: model_dict
        """
        # 构建模型字段的字典
        model_dict = {field: row[self.index_list[i]] for i, field in enumerate(self.table_fields)}

        if self.additional_field_name:
            # 获取不在模型中的附加字段
            model_dict[self.additional_field_name] = {self.title[i]: row[i] for i in range(len(row)) if i not in self.index_list}

        return model_dict

    def get_model_data(self, row):
        """
        获取 Django 模型使用的字典数据
        """
        return self.row_to_dict(row)


class ModelData:
    """
    根据 django 模型转换导入数据

    例子：
    reader = ['rows', 'rows']
    iter_reader = iter(reader)
    model = '数据模型'
    conversion_type = '需要转换格式的字段名'
    export = ModelData(model, next(iter_reader))
    load_list = []
    # 如果需要转换类型
    if isinstance(conversion_type, dict):
        export.conversion_type = conversion_type
    for row in iter_reader:
        # print(ku_field, xls_row, row)
        kwargs = export.get_import_data(row)
        # print(kwargs)
        load_list.append(model(**kwargs))
    model.objects.bulk_create(load_list)
    """

    # 获取字段类型，处理一部分类型
    WIDGETS_MAP = {
        # 'ManyToManyField': 'get_m2m_widget',
        # 'OneToOneField': 'get_fk_widget',
        # 'ForeignKey': 'get_fk_widget',
        # 'DecimalField': widgets.DecimalWidget,
        # 'DateTimeField': widgets.DateTimeWidget,
        # 'DateField': widgets.DateWidget,
        # 'TimeField': widgets.TimeWidget,
        # 'DurationField': widgets.DurationWidget,
        # 'FloatField': float,
        # 'IntegerField': widgets.IntegerWidget,
        # 'PositiveIntegerField': widgets.IntegerWidget,
        # 'BigIntegerField': widgets.IntegerWidget,
        # 'PositiveSmallIntegerField': widgets.IntegerWidget,
        # 'SmallIntegerField': widgets.IntegerWidget,
        # 'AutoField': widgets.IntegerWidget,
        # 'NullBooleanField': widgets.BooleanWidget,
        # 'BooleanField': widgets.BooleanWidget,
    }

    def __init__(self, model, row, import_names=None, exclude=None):

        # 需要转换的 id 然后批量转换
        # 如果有需要 再添加

        # print(filed)

        if exclude is None:
            exclude = []

        self.model = model
        self.import_fields = []
        self.import_index = []
        self.conversion_type = {}
        # 清理字段，有些字段不能为空字符，我们在这里清理
        self.clean_fields = set()

        names = {}
        verbose_names = {}
        if import_names is None:
            import_names = {}

        for f in model._meta.fields:
            # print('处理不同字段内容', isinstance(f, models.BooleanField))
            # print('获取模型的字段名', f.get_internal_type())
            internal_type = f.get_internal_type() if callable(getattr(f, "get_internal_type", None)) else ""
            # print(internal_type, f.verbose_name, f.name)
            names[f.name] = {'name': f.name, 'internal_type': internal_type}
            verbose_names[str(f.verbose_name)] = {'name': f.name, 'internal_type': internal_type}

        for i, field in enumerate(row):
            field = field.strip()
            # 排除的内容
            if field in exclude:
                continue

            # 重命名的内容
            if field in import_names:
                field = import_names[field]

            # 字段名导入
            if field in names:
                self.import_fields.append(names[field]['name'])
                self.import_index.append(i)
                if names[field]['internal_type'] in self.WIDGETS_MAP:
                    self.conversion_type[names[field]['name']] = self.WIDGETS_MAP[names[field]['internal_type']]
            # 按名字导入
            elif field in verbose_names:
                self.import_fields.append(verbose_names[field]['name'])
                self.import_index.append(i)
                if verbose_names[field]['internal_type'] in self.WIDGETS_MAP:
                    self.conversion_type[verbose_names[field]['name']] = self.WIDGETS_MAP[
                        verbose_names[field]['internal_type']]
            # 没有导入的内容
            else:
                # pass
                # print('导入失败：-------------------表头 %s %s 应该为 %s 。-------------------' % (i + 1, field, now_0[i]))
                # print(field, field in names, names, )
                # print(field, field in verbose_names, verbose_names, )
                raise RuntimeError('导入失败：row[%s]："%s" 不能导入数据库。' % (i + 1, field))

    def get_import_data(self, row):
        """
        """
        import_data = {}

        for index, field in zip(self.import_index, self.import_fields):
            if field in self.clean_fields:
                if row[index] == 0:
                    ...
                elif not row[index]:
                    continue
            import_data[field] = row[index]

        for field, func in self.conversion_type.items():
            # print('转换类型', self.conversion_type[field], val)
            import_data[field] = func(import_data[field])

        return import_data


def get_field_file_path(field_file):
    """
    获取 模型 FileField 路径
    """
    # print(_fieldfile.path)
    # print(_fieldfile.upfile.name)
    # print(_fieldfile.read())
    try:
        return field_file.path
    except:
        return None


def get_queryset_sum(queryset, field_list, *args):
    """
    计算 QuerySet 的和 参数 QuerySet 需要求和的字段  需要显示的字段
    """

    kwargs = {}

    try:
        # 获取需要求和的内容
        for i in field_list:
            # print(i)
            if '_quanli_' in i:
                # print('_quanli_')
                continue
            elif i.endswith('_int') or i.endswith('_float'):
                if 'rmb_int' in i:
                    pass
                elif 'fencheng_int' in i:
                    pass
                else:
                    # print(i)
                    kwargs[i] = Sum(i)
        if kwargs:
            return queryset.values(*args).annotate(**kwargs)
        else:
            return None
    except:
        pass
    return None


def calc_sum(query_set, field_list):
    """
    计算 QuerySet 中指定字段的和

    这个和 get_queryset_sum 不同，get_queryset_sum 是按分组求和，calc_sum 是聚合求和

    :param query_set: 数据集
    :param field_list: 需要求和的字段
    :return: 求和字段的字典
    """
    assert isinstance(query_set, QuerySet)
    return query_set.aggregate(**{field: Sum(field) for field in field_list})


def annotate(query_set, fields, func='Count', values_fields=None):
    """
    数据聚合

    :param query_set: 查询集
    :param fields: 字段列表
    :param func: 聚合函数 Aggregate, Avg, Count, Max, Min, StdDev, Sum, Variance
    :param values_fields: 用来分组显示的字段名
    :return: 聚合内容
    """
    assert isinstance(query_set, QuerySet)

    if values_fields is None:
        values_fields = fields

    functions = {
        'Aggregate': Aggregate,
        'Avg': Avg,
        'Count': Count,
        'Max': Max,
        'Min': Min,
        'StdDev': StdDev,
        'Sum': Sum,
        'Variance': Variance,
    }
    return query_set.order_by(*values_fields).values(*values_fields).annotate(
        *[functions[func](field) for field in fields])


def group_by(query_set, group_field):
    """
    util:django 获取分类列表 ( 对某些取到的 QuerySet 分组)

    例如获取厂牌名称
    print(group_by(objs,'shouquan_changpai'))
    》 ['厂牌名称1', '厂牌名称2', ...]

    :param query_set:
    :param group_field: 需要分组的字段
    :return: 分组字段的内容列表
    """

    django_groups = annotate(query_set, [group_field], func='Count')
    # django_groups = query_set.values(group_field).annotate(Count(group_field))
    groups = []
    for dict_ in django_groups:
        # print(dict_)
        groups.append(dict_.get(group_field))
    return groups


def calc_file_size(objs, field='file_size'):
    """
    从 QuerySet 计算文件大小
    """
    return objs.exclude(**{field: None}).aggregate(**{field: Sum(field)})[field]


def get_date_range(queryset, date_field='shujuriqi_date', format_string="%Y-%m-%d"):
    """
    获取 QuerySet 的日期间隔 参数 queryset，日期的字段
    """

    from_date = queryset.order_by(date_field)[:1].values_list(date_field)[0][0].strftime(format_string)
    to_date = queryset.order_by('-' + date_field)[:1].values_list(date_field)[0][0].strftime(format_string)
    return from_date, to_date


def confirm_db(db_name=None, is_sqlite3=True, db_name_list=None, title='确认是否继续修改数据库', message='开始修改数据库'):
    """
    确认是否修改线上数据库
    本地操作，因为如果没有修改会直接退出，防止误操作

    :param db_name: 数据库名字，优先（sqlite3）参数处理
    :param is_sqlite3: 如果是 sqlite3 数据库 直接修改数据
    :param db_name_list: 可以直接修改的数据库名字列表
    :param title: 提示标题
    :param message: 确认后的提示信息
    :return:
    """

    if db_name_list is None:
        db_name_list = []

    name = settings.DATABASES['default']['NAME']
    if db_name is not None and db_name == name:
        return
    elif name in db_name_list:
        return
    elif is_sqlite3 and settings.DATABASES['default']['ENGINE'] == 'django.db.backends.sqlite3':
        return

    try:
        import wx
        app = wx.App()
        dlg = wx.MessageDialog(None, f'正在修改数据库({name})，是否继续？', title, wx.YES_NO | wx.ICON_QUESTION)
        if dlg.ShowModal() == wx.ID_YES:
            print(f'{Fore.MAGENTA}{message}({name}){Style.RESET_ALL}')
            return
    except ModuleNotFoundError:
        from djlds.util import confirm_yes_no
        if confirm_yes_no(title=title, text=f'正在修改数据库({name})，是否继续？\n输入 y 确认，其他任意字符表示取消'):
            print(f'{Fore.MAGENTA}{message}({name}){Style.RESET_ALL}')
            return
    exit(f'退出程序，不修改数据库({name})！')


def sync_model(model, src, dst, fields=None, exclude=None, is_clear=False):
    """
    同步模型数据
    :param model:
    :param src:
    :param dst:
    :param fields:
    :param exclude:
    :param is_clear:
    :return:
    """

    load_list = []

    for obj in model.objects.using(src).all():
        kwargs = model_to_dict(obj, fields=fields, exclude=exclude)
        load_list.append(model(**kwargs))

    if load_list:
        if is_clear:
            print('删除数据', model.objects.using(dst).all().delete())
        print('成功导入 %s 行' % len(model.objects.using(dst).bulk_create(load_list)))
    else:
        print('没有数据导入！')



import os
import sys
import subprocess
from importlib import import_module

VERSION = '3.1'

CONFIG = {
    'USERNAME': os.getenv('USERNAME', 'lds'),
    'EMAIL': os.getenv('EMAIL', '85176878@qq.com'),
    'SCRIPTS_PATH': os.getenv('SCRIPTS_PATH', r'D:\Envs\lds_mysite_310\Scripts'),
    'BACKUP_DIR': 'backup',
    'DJANGO_SETTINGS_MODULE': os.getenv('DJANGO_SETTINGS_MODULE', 'mysite.settings'),
    'SERVER_ADDRESS': '127.0.0.1',
    'SERVER_PORT': '8100'
}


def run_command(command, capture_output=False, show_command=True):
    if show_command:
        print(f'运行命令：{command}')
    result = subprocess.run(command, shell=True, text=True, capture_output=capture_output)
    if capture_output:
        return result.stdout.strip() if result.returncode == 0 else None
    return result.returncode == 0


class DjangoTool:

    def __init__(self, config):
        self.config = config
        self.working_dir = os.getcwd()
        self.python = os.path.join(self.config['SCRIPTS_PATH'], 'python.exe')
        self.actions = {
            'r': self.runserver,
            's': self.create_app,
            '1': self.create_user,
            '2': self.check_db,
            '3': self.create_db,
            'b': self.dbbackup,
            'x': self.dbrestore,
            'm': self.mediabackup,
            'a': self.mediarestore,
            'e': self.export_requirements,
            'i': lambda: self.import_requirements("requirements.txt"),
            'd': lambda: self.import_requirements("requirements_dev.txt"),
        }

    def set_cmd_title(self, title):
        if os.name == 'nt':
            os.system(f'title {title}')
        else:
            sys.stdout.write(f'\33]0;{title}\a')
            sys.stdout.flush()

    def clear_screen(self):
        os.system('cls' if os.name == 'nt' else 'clear')

    def pause(self, message="按回车返回菜单..."):
        input(message)

    def list_backup_files(self):
        if not os.path.exists(self.config['BACKUP_DIR']):
            print(f"备份目录 '{self.config['BACKUP_DIR']}' 不存在！")
            return []
        return [f for f in os.listdir(self.config['BACKUP_DIR']) if os.path.isfile(os.path.join(self.config['BACKUP_DIR'], f))]

    def get_databases_from_settings(self):
        sys.path.append(self.working_dir)
        try:
            settings = import_module(self.config['DJANGO_SETTINGS_MODULE'])
            return list(settings.DATABASES.keys()), settings.DATABASES
        except Exception as e:
            print(f"加载Django设置时出错: {e}")
            return [], {}

    def ensure_virtual_env(self):
        if not os.path.exists(self.config['SCRIPTS_PATH']):
            print(f"Django Tool {VERSION} 版")
            print(f"虚拟环境的路径不存在，请修正文件路径（{self.config['SCRIPTS_PATH']}）或新建虚拟环境。")
            input("将退出命令行工具，按回车退出...")
            exit()

    def ensure_manage_py(self):
        if not os.path.exists(os.path.join(self.working_dir, 'manage.py')):
            print("错误: manage.py 没找到。请在Django项目根目录中运行此脚本。")
            sys.exit(1)

    def get_version_info(self):
        python_version = run_command(f"{self.python} --version", capture_output=True, show_command=False)
        django_version = run_command(f"{self.python} -m django --version", capture_output=True, show_command=False)
        return python_version, django_version

    def print_menu(self):
        self.clear_screen()
        print('')
        print(f"  Django Tool {VERSION}")
        print(f"  使用环境：{self.config['SCRIPTS_PATH']}\\activate.bat")
        print(f"  工作目录：{self.working_dir}")
        print("\n   选项   菜单\n")
        print("   [r]    启动服务器")
        print("   [s]    创建应用程序")
        print("   [1]    创建管理员")
        print("   [2]    检查迁移")
        print("   [3]    迁移数据")
        print("   [b]    备份数据库 (dbbackup)")
        print("   [x]    恢复数据库 (dbrestore)")
        print("   [m]    备份媒体文件 (mediabackup)")
        print("   [a]    还原媒体文件 (mediarestore)")
        print("   [e]    导出 requirements.txt")
        print("   [i]    安装 requirements.txt")
        print("   [d]    安装 requirements_dev.txt")
        print("\n   输入其他任意内容退出...\n")

    def select_choice(self):
        return input("请输入 [] 内的选项，按回车：").strip().lower()

    def execute_action(self, action):
        if action:
            action()
        else:
            print("选择无效，将退出...")

    def runserver(self):
        print(f"\n启动服务器，管理地址：{self.config['SERVER_ADDRESS']}:{self.config['SERVER_PORT']}/admin\n")
        run_command(f"{self.python} manage.py runserver {self.config['SERVER_ADDRESS']}:{self.config['SERVER_PORT']}")

    def create_app(self):
        app_name = input("输入应用程序的名称：").strip()
        if app_name:
            run_command(f"{self.python} manage.py startapp {app_name}")

    def create_user(self):
        run_command(f"{self.python} manage.py migrate")

        input_username = input(f"输入用户名（默认值：{self.config['USERNAME']}）：").strip() or self.config['USERNAME']
        input_email = input(f"输入邮箱（默认值：{self.config['EMAIL']}）：").strip() or self.config['EMAIL']

        print(f"\n将使用用户名：{input_username} 和邮箱：{input_email}\n")
        run_command(f"{self.python} manage.py createsuperuser --username={input_username} --email={input_email}")

    def check_db(self):
        run_command(f"{self.python} manage.py check")

    def create_db(self):
        app_name = input("为应用程序迁移：").strip()
        if app_name:
            run_command(f"{self.python} manage.py makemigrations {app_name}")
        else:
            run_command(f"{self.python} manage.py makemigrations")
        run_command(f"{self.python} manage.py migrate")
        print("\n迁移数据完成\n")

    def dbbackup(self):
        print("\n备份数据库\n")
        if run_command(f"{self.python} manage.py dbbackup"):
            print("\n数据库备份完成\n")
        else:
            print("\n数据库备份失败\n")

    def dbrestore(self):
        print("\n恢复数据库\n")

        databases, db_settings = self.get_databases_from_settings()
        if not databases:
            print("没有找到数据库！")
            return

        print("可用的数据库：")
        for i, db in enumerate(databases):
            print(f"  [{i}] {db}")

        db_choice = input("请输入要恢复的数据库编号（默认选择[0]）：").strip()
        db_choice = int(db_choice) if db_choice.isdigit() and 0 <= int(db_choice) < len(databases) else 0
        database = databases[db_choice]

        backup_files = self.list_backup_files()
        if not backup_files:
            print("没有找到备份文件！")
            return

        print("可用的备份文件：")
        for i, filename in enumerate(backup_files):
            print(f"  [{i}] {filename}")

        file_choice = input("请输入备份文件的编号进行恢复（默认选择[0]）：").strip()
        file_choice = int(file_choice) if file_choice.isdigit() and 0 <= int(file_choice) < len(backup_files) else 0

        if input("是否需要清空数据库？(y/n，默认n)：").strip().lower() == 'y':
            self.clear_and_migrate_database(database, db_settings)

        if run_command(f"{self.python} manage.py dbrestore -i \"{backup_files[file_choice]}\" --database={database}"):
            print(f"\n已从文件 '{backup_files[file_choice]}' 恢复数据库 '{database}'\n")
        else:
            print(f"\n数据库 {backup_files[file_choice]} 恢复失败\n")

    def clear_and_migrate_database(self, database, db_settings):
        print("清空数据库中...")
        db_setting = db_settings[database]
        if db_setting['ENGINE'] == 'django.db.backends.sqlite3':
            db_path = db_setting['NAME']
            if os.path.exists(db_path):
                os.remove(db_path)
            print("已删除SQLite数据库文件。")
        else:
            if not run_command(f"{self.python} manage.py flush --no-input --database={database}"):
                print("\n清空数据库失败，请手动处理。\n")

        print("重新迁移数据库中...")
        if not run_command(f"{self.python} manage.py migrate --database={database}"):
            print("\n数据库迁移失败，请手动处理。\n")

    def mediabackup(self):
        print("\n备份媒体文件\n")
        if run_command(f"{self.python} manage.py mediabackup"):
            print("\n媒体文件备份完成\n")
        else:
            print("\n媒体文件备份失败\n")

    def mediarestore(self):
        print("\n恢复媒体文件\n")
        backup_files = self.list_backup_files()
        if not backup_files:
            print("没有找到备份文件！")
            return

        print("可用的备份文件：")
        for i, filename in enumerate(backup_files):
            print(f"  [{i}] {filename}")

        file_choice = input("请输入备份文件的编号进行恢复（默认选择[0]）：").strip()
        file_choice = int(file_choice) if file_choice.isdigit() and 0 <= int(file_choice) < len(backup_files) else 0

        if run_command(f"{self.python} manage.py mediarestore -i \"{backup_files[file_choice]}\""):
            print(f"\n已从文件 '{backup_files[file_choice]}' 恢复媒体文件\n")
        else:
            print(f"\n媒体文件 {backup_files[file_choice]} 恢复失败\n")

    def export_requirements(self):
        run_command(f"{os.path.join(self.config['SCRIPTS_PATH'], 'pip.exe')} freeze > requirements.txt")
        print("\n导出 requirements.txt 完成\n")

    def import_requirements(self, requirements_file):
        use_custom_mirror = input("是否使用 https://pypi.douban.com/simple/ 进行安装？(y/n, 默认n)：").strip().lower() == 'y'
        command = f"{os.path.join(self.config['SCRIPTS_PATH'], 'pip.exe')} install -r {requirements_file}"
        if use_custom_mirror:
            command += " -i https://pypi.douban.com/simple/"
        run_command(command)
        print(f"\n安装 {requirements_file} 完成\n")

    def menu(self):
        while True:
            self.print_menu()
            choice = self.select_choice()
            action = self.actions.get(choice)
            self.execute_action(action)

            if not action:
                break
            self.pause()

    def run(self):
        python_version, django_version = self.get_version_info()
        self.set_cmd_title(f"Django 管理工具 {VERSION} [{python_version}] [Django {django_version}]")
        self.ensure_virtual_env()
        self.ensure_manage_py()
        os.environ.setdefault('DJANGO_SETTINGS_MODULE', self.config['DJANGO_SETTINGS_MODULE'])
        self.menu()


def main(config):
    check_config = [key for key in CONFIG.keys() if key not in config]
    if check_config:
        raise ValueError(f'config 中缺少设置内容:{check_config}')
    dt = DjangoTool(config)

    dt.run()


if __name__ == "__main__":
    main(CONFIG)

# 下面是 djt.py 示例：
"""
import os
import sys
import subprocess

CONFIG = {
    'USERNAME': os.getenv('USERNAME', 'lds'),
    'EMAIL': os.getenv('EMAIL', '85176878@qq.com'),
    'SCRIPTS_PATH': os.getenv('SCRIPTS_PATH', r'D:\Envs\lds_mysite_310\Scripts'),
    'BACKUP_DIR': 'backup',
    'DJANGO_SETTINGS_MODULE': os.getenv('DJANGO_SETTINGS_MODULE', 'mysite.settings'),
    'SERVER_ADDRESS': '127.0.0.1',
    'SERVER_PORT': '8100'
}


def execute_script_with_alternative_python(required_python_path):
    required_python_executable = os.path.join(required_python_path, 'python.exe')
    if os.path.normcase(os.path.normpath(required_python_executable)) != os.path.normcase(os.path.normpath(sys.executable)):
        print(f"当前 Python 路径“{sys.executable}”与所需路径“{required_python_executable}”不同")
        print(f"切换到 Python: {required_python_executable}")
        subprocess.run([required_python_executable, sys.argv[0]], check=True)
        sys.exit()


def main():
    # 确保使用正确的 Python 路径
    execute_script_with_alternative_python(CONFIG['SCRIPTS_PATH'])
    from djlds.tool import main as djlds_main
    djlds_main(CONFIG)


if __name__ == "__main__":

    try:
        main()
    except Exception as e:
        print(f"运行错误: {e}")
        input('输入任意字符退出...')
"""



__package_name__ = 'django-admin-generator'
__version__ = '2.2.0'
__author__ = 'Rick van Hattem'
__author_email__ = 'Wolph@Wol.ph'
__description__ = ' '.join(('''
Django Admin Generator 是一个管理命令，用于为给定的应用程序/模型自动生成 Django `admin.py` 文件。
Usage
为给定应用程序生成 admin.py：
./manage.py admin_generator APP_NAME >> APP_NAME/admin.py

要使用给定的应用程序（以用户开头）为所有模型生成管理员，请执行以下操作：
./manage.py admin_generator APP_NAME ‘^user’ >> APP_NAME/admin.py
'''.strip().split()))
__url__ = 'https://github.com/WoLpH/django-admin-generator/'




# ---------------------------------------
#   程序：import_export.py
#   版本：0.11
#   作者：lds
#   日期：2023-09-22
#   语言：Python 3.X
#   说明：django 导入和导出
# ---------------------------------------

import csv
import time
from pathlib import Path

from django.db import transaction
# 获取模型实例的字典
from django.forms.models import model_to_dict
from ilds.excel_xlrd import ReadXlsx as ReadXls
from ilds.excel_xlsx import ReadXlsx

from djlds.model import ModelFields, get_field_file_path, ModelData, TableData


class BaseImportExcel:
    """
    导入 Excel 的基础类

    我们没有使用已经写好的导入库，是为了速度和方便定制。
    """

    def __init__(self, model, exclude=None, revised=None, debug=False):
        self.file = None
        self.model = model
        self.read = None  # 正在读取的数据，有时候我们会用到数据列表以外的数据
        self.debug = debug
        if exclude is None:
            self.exclude = self.get_exclude()
        else:
            self.exclude = exclude
        if revised is None:
            self.revised = self.get_revised()
        else:
            self.revised = revised
        self.table = TableData(model)
        self.field_data = []
        self.init_parsing_field()

        self.titles = None
        self.is_multiple_sheet = False

        self.count = 0
        self.count_all = 0
        self.load_list = []
        self.adding_data = {}  # 需要附加数据的时候使用
        self.model_kwargs = {}  # 创建数据的时候添加公用数据
        # 导入数据用的时间
        self.time_cost = 0
        self.info = []

    def init_parsing_field(self):
        """
        初始化处理字段内容
        """
        # super().init_parsing_field()
        parsing_field = {
            'IntegerField': self.parsing_integer,
            'FloatField': self.parsing_float,
            'DateField': self.parsing_date,
            'DateTimeField': self.parsing_date_time,
            'BooleanField': self.parsing_boolean,
        }
        for data in self.table.iter('field', 'type'):
            _field, _type = data
            # print(data)
            if _type in parsing_field:
                self.field_data.append((_field, parsing_field[_type]))
        # pp(self.field_data)

    def parsing_integer(self, field, data):
        """
        解析数字字段
        """
        if field in data:
            try:
                data[field] = int(data[field])
            except Exception as e:
                if self.debug:
                    info = f'parsing_integer: {field} {e}'
                    print(info)
                    self.info.append(info)
                del data[field]

    def parsing_float(self, field, data):
        """
        解析浮点数字段
        """
        if field in data:
            try:
                data[field] = float(data[field])
            except Exception as e:
                if self.debug:
                    print(f'parsing_float: {field} {e}')
                del data[field]

    def parsing_date(self, field, data):
        """
        解析日期字段
        """
        if field in data:
            v = data[field]
            if not v:
                del data[field]
            elif '/' in v:
                data[field] = v.replace('/', '-')

    def parsing_date_time(self, field, data):
        """
        解析时间字段
        """
        if field in data:
            v = data[field]
            if not v:
                del data[field]
            elif '/' in v:
                data[field] = v.replace('/', '-')

    def parsing_boolean(self, field, data):
        """
        解析布尔字段
        """
        if field in data:
            v = str(data[field]).lower()
            if v in ['true', '有', '是', ]:
                data[field] = True
            elif v in ['false', '无', '否', ] or not v:
                data[field] = False
            elif v in ['none', '未知', ]:
                data[field] = None

    def get_revised(self):
        """
        需要修正的标题
        """
        return {}

    def get_exclude(self):
        """
        排除的标题
        """
        return []

    def init_title(self, datasets):
        """
        初始化标题
        """
        # # 第一行不是标题的例子
        # max_row = 20
        # for i, row in enumerate(datasets):
        #     # print(i, row)
        #     # 检查是否包含指定内容
        #     if "账号昵称（必填）" in row and "发布链接（必填）" in row:
        #         self.titles = row
        #         break
        #     if i >= max_row:
        #         break
        #
        # if not self.titles:
        #     return None

        self.titles = next(datasets)
        ret = self.table.set_title(self.titles, exclude=self.exclude, **self.revised)
        # print('set_title', self.titles, self.exclude, self.revised)
        if any([data['name'] for data in ret]):
            print([data['name'] for data in ret])
            raise ValueError(f'没有匹配的字段：{ret}')
        if self.table.duplicate_info:
            raise ValueError(f'重复的字段：{self.table.duplicate_info}')
        if self.debug:
            print(self.table.table_fields)
            print(self.table.index_list)

    def init(self):
        """
        多表导入的时候，初始化数据
        """
        self.count = 0
        self.titles = None
        self.load_list = []

    def init_handle(self):
        """
        初始化处理数据需要的前置内容
        """
        ...

    def handle_data(self, data):
        """
        处理数据
        """
        for field, fun in self.field_data:
            fun(field, data)
        return data

    def ignore_import(self, kwargs):
        """
        根据条件来判断是否忽略导入
        """
        # print(kwargs)
        if not any(kwargs.values()):
            return True

    def import_data(self, datasets):
        """
        导入数据
        """

        self.init_handle()

        for i, data in enumerate(datasets):
            # print(data)
            kwargs = self.table.get_model_data(data)
            if self.ignore_import(kwargs):
                if self.debug:
                    print(f'跳过行 {i}：{data}')
                continue

            kwargs.update(self.model_kwargs)

            kwargs = self.handle_data(kwargs)

            try:
                self.append(kwargs, data)
            except Exception as e:
                raise ValueError(f"i:{i} 错误: {e}\ntitles: {self.titles}\ndata: {data}\nkwargs: {kwargs}")

        if not self.debug:
            self.count += len(self.model.objects.bulk_create(self.load_list))

        self.info.append(f'{self.read.title} 导入 {self.count} 行')
        self.count_all += self.count

        return self.info

    def append(self, data, original_data):
        # super().append(data, original_data)
        if self.debug:
            self.model.objects.create(**data)
            self.count += 1
        else:
            self.load_list.append(self.model(**data))

    def read_file(self, file, *args, **kwargs):
        """
        读取文件
        """
        self.file = Path(file)
        suffix = self.file.suffix.lower()
        if suffix == '.xlsx':
            self.read = ReadXlsx(file, *args, **kwargs)
            datasets = None
        elif suffix == '.xls':
            self.read = ReadXls(file, *args, **kwargs)
            datasets = None
        elif suffix == '.csv':
            if 'encoding' in kwargs:
                encoding = kwargs.pop('encoding')
            else:
                encoding = 'utf-8-sig'
            # 读取csv文件
            f = open(file, newline='', encoding=encoding)
            self.read = csv.reader(f, *args, **kwargs)  # delimiter=':', quoting=csv.QUOTE_NONE
            datasets = iter(self.read)
            if self.is_multiple_sheet:
                print('cvs 文件没有多个表')
                self.is_multiple_sheet = False
        else:
            raise ValueError(f'导入 {file} 失败， 不支持导入 {suffix[1:]} 格式')

        return datasets

    @transaction.atomic
    def start(self, file, exclude_sheet=None, *args, **kwargs):
        """
        开始导入数据
        @param file:
        @param exclude_sheet: 忽略导入的表
        @return:
        """

        start_time = time.time()

        if exclude_sheet is None:
            exclude_sheet = []

        datasets = self.read_file(file, *args, **kwargs)

        for sheet_name in self.read.sheet_names:
            print('sheet_name', sheet_name)
            self.read.set_sheet(sheet_name)

            if datasets is None:
                datasets = self.read.values()

            # 跳过结算数据导入
            if self.read.title in exclude_sheet:
                info = f'排除导入 {self.read.title}'
                if self.debug:
                    print(info)
                self.info.append(info)
            else:
                self.init_title(datasets)
                if self.titles:
                    self.import_data(datasets)
                else:
                    info = f'跳过导入“{self.read.title}”，没有找到标题'
                    if self.debug:
                        print(info)
                    self.info.append(info)

            if not self.is_multiple_sheet:
                break

            self.init()

        self.time_cost = time.time() - start_time

        info = f'总共导入 {self.count_all} 条数据'
        print(info)
        self.info.append(info)

        self.end_import()

        return self.info

    def end_import(self, *args, **kwargs):
        """
        结束导入数据的时候调用
        """
        ...


def import_csv(model, csv_file, encoding='utf-8', conversion_type=None, max_batch=10000, import_names=None,
               exclude=None):
    """
    导入 csv 文件到数据库

    注意：需要处理日期和文件名的不要用她导入，她只用于备份和恢复。

    :param model: 要导出的数据库模型
    :param csv_file: 保存文件
    :param encoding: 保存文件编码
    :param conversion_type: 转换字段数据内容
    :param max_batch: 一次最大导入数量，超过以后分批导入
    :param import_names: 重命名标题名，比如 厂牌名称 改名为 版权公司名
    :param exclude: 排除导入的内容
    :return:
    """

    print('导入开始 ---------------------------------------------------------\n', csv_file)

    # 读取csv文件
    with open(csv_file, newline='', encoding=encoding) as f:
        reader = csv.reader(f)
        import_count = 0
        batch_count = 0
        load_list = []

        try:
            iter_reader = iter(reader)
            export = ModelData(model, next(iter_reader), import_names, exclude)
            export.clean_fields.add('shouquan_kaishi_date')
            export.clean_fields.add('shouquan_jieshu_date')
            export.clean_fields.add('external_settlement_float')
            # 如果需要转换类型
            if isinstance(conversion_type, dict):
                for k, v in conversion_type.items():
                    # print(k, v, export.import_fields)
                    if k in export.import_fields:
                        export.conversion_type[k] = v
            for row in iter_reader:
                # print(ku_field, xls_row, row)
                kwargs = export.get_import_data(row)
                # print(kwargs)
                load_list.append(model(**kwargs))
                batch_count += 1
                if batch_count == max_batch:
                    import_count += len(model.objects.bulk_create(load_list))
                    batch_count = 0
                    load_list = []
        except csv.Error as e:
            return f'file {csv_file}, line {reader.line_num}: {e}'
        else:
            import_count += len(model.objects.bulk_create(load_list))
            info = '成功导入 %s 行' % import_count
            print(info)
            print('---------------------------------------------------------')
            return info


class LdsExportCsv:
    """
    写入数据到 csv 文件
    支持自定义写入的表头和要导出的字段

    这个主要是自用，等抽时间我们改成通用的

    例子

    # model = 要导出的模型
    objs = model.objects.all()
    # 找到的歌曲列表
    if objs.exists():
        xlsxfile = r'导出：测试.csv'
        ex_csv = LdsExportCsv(model, xlsxfile)  # 歌曲库的中文名
        ex_csv.set_title()
        # 或者元组列表
        # ex_csv.set_title(['歌曲名称', '表演者', '专辑名称',] ) # , row=3, col=3
        for obj in objs:
            ex_csv.append(obj)
        info = ex_csv.print_count_info()
        ex_csv.close()
    else:
        print('没有找到歌曲')
    """

    def __init__(self, model, work_file, exclude=None, encoding='utf-8'):
        """初始化"""
        self.title = []
        self.export = None

        # 要处理的库
        self.model = model

        # 写入csv文件
        self.csv_file = open(work_file, 'w', newline='', encoding=encoding)  # 有软件打开乱码
        self.writer_csv = csv.writer(self.csv_file)

        self.gequ_dict = {}

        # 设置排除内容
        if exclude is None or not isinstance(exclude, list):
            self.exclude = []  # 'id',
        else:
            self.exclude = exclude

        self.f = ModelFields(model, self.exclude)

        # 初始化统计
        self.count_all = 0
        self.info = {}
        self.count_merge = 0

        self.field_list = []

    def set_title(self, fields=None):
        """
        设置标题和写入的内容
        export 是要导出列的名字列表
        """

        # 需要获取数据的字段列表
        self.field_list = []
        if fields is None:
            self.export = self.f.verbose_list
        else:
            self.export = fields

        # 获取模型的字段
        model_field = self.f.verbose_dict()
        # pprint(model_field)

        # 写表头
        csv_row = []
        for i_export in self.export:
            i_export = i_export.strip()
            if i_export:
                try:
                    self.field_list.append(model_field[i_export])
                except:
                    self.field_list.append('None')
                # print(self.field_list)
                csv_row.append(i_export)
        self.writer_csv.writerow(csv_row)

        self.init_count()

        return

    def append(self, date, row=0, col=0):
        """
        写入 csv 文件
        date 要写入的数据
        """

        model_dict = model_to_dict(date, exclude=self.exclude)

        # 写入文件
        # print(model_dict)
        # print(self.field_list)
        csv_row = []
        for i_field_list in self.field_list:
            try:
                # 统计信息
                if i_field_list in self.info:
                    value = model_dict[i_field_list]
                    if i_field_list.endswith('_float'):
                        self.info[i_field_list] += float(value)
                    else:
                        self.info[i_field_list] += int(value)
                    # print(self.info)
                # 处理日期
                # if i_field_list == 'shujuriqi_date':
                #     value = model_dict[i_field_list].strftime("%Y-%m")
                if i_field_list.endswith('_date'):
                    value = model_dict[i_field_list].strftime("%Y-%m-%d")
                # 处理文件
                elif i_field_list.endswith('_file'):
                    value = get_field_file_path(model_dict[i_field_list])
                else:
                    value = model_dict[i_field_list]
            except:
                value = ''
            csv_row.append(value)
        self.writer_csv.writerow(csv_row)

        # print(model_dict)
        self.count_all += 1
        return

    def get_count_list(self):
        """获取统计信息的列表 列表内容是字典 键为 count_name count_field count"""
        count_list = []
        model_fields = self.f.verbose_dict()
        # print(model_fields)
        if self.info:
            for k, v in model_fields.items():
                _dict = {}
                _field = self.info.get(v)
                # print(k,v,_field)
                if not _field is None:
                    _dict['count_name'] = k
                    _dict['count_field'] = v
                    _dict['count'] = self.info[v]
                    count_list.append(_dict)
        return count_list

    def print_count_info(self):
        """打印统计信息"""
        print_info = '导出：%s    总共：%s' % (self.count_all - self.count_merge, self.count_all)
        for _count in self.get_count_list():
            print_info += '%s：%s    ' % (_count['count_name'], _count['count'])  # _count['count_field']
        print(print_info)
        return print_info

    def init_count(self):
        """ 初始化需要统计的内容"""

        self.info = {}
        # print(field_list)
        for i in self.field_list:
            if '_quanli_' in i:
                # print('_quanli_')
                continue
            elif i.endswith('_int') or i.endswith('_float'):
                if 'rmb_int' in i:
                    pass
                elif 'fencheng_int' in i:
                    pass
                else:
                    # print(i)
                    self.info[i] = 0

        # print(self.info)

    def close(self):
        """
        写入结束，关闭文件
        """
        self.csv_file.close()

    def __repr__(self):
        return str(self.export)


def export_csv(model, sep='', max_export=1000000, exclude=None):
    """
    导出数据为csv文件
    """

    name_str = '导出 %s' % model.__name__
    # name_str = '导出 %s(%s)' % (ku_str, ku.__name__)

    if exclude is None:
        exclude = []

    data_list = model.objects.all()
    # 自定义了到处方式
    # # 按日期导出
    # kwargs = {}
    # kwargs['shujuriqi_date__range'] = ['2018-01-01', '2100-01-01']
    # data_list = ku.objects.filter(**kwargs)
    try:
        data_count = data_list.count()
    except Exception as e:
        return f'{model.__name__} 没有歌曲导出 {e}'

    # 找到的歌曲列表
    if data_count:
        print('---------------------------------------------------------')
        print('开始 %s' % name_str)
        print()

        print('搜索结果：', data_count)
        if data_count > max_export:
            export_start = 0
            export_stop = max_export
            info = '已分多个文件导出：'
            # print(info)
            while True:
                if export_start > data_count:
                    break
                data_list = model.objects.all()[export_start:export_stop]
                if export_stop == max_export:
                    name = name_str + '.csv'
                else:
                    # print(type(export_start),type(max_export),type(export_start/max_export))
                    name = name_str + '_%03d.csv' % int(export_start / max_export)
                mycsv = LdsExportCsv(model, name, exclude=exclude)  # 歌曲库的中文名
                mycsv.set_title()  # , row=3, col=3
                for i_data_list in data_list:
                    # print(i_data_list)
                    mycsv.append(i_data_list)
                info += sep + mycsv.print_count_info()
                mycsv.close()
                export_start += max_export
                export_stop += max_export
        else:
            print('没有超过 %s' % max_export)
            mycsv = LdsExportCsv(model, name_str + '.csv', exclude=exclude)  # 歌曲库的中文名
            mycsv.set_title()  # , row=3, col=3
            for i_data_list in data_list:
                # print(i_data_list)
                mycsv.append(i_data_list)
            mycsv.close()
            info = mycsv.print_count_info()
            # print(mycsv.tongji_info)
            # mycsv.print_count_info()
            print()
            print('导出结束')
            print('---------------------------------------------------------')
    else:
        info = (model.__name__, '库没有歌曲')

    # print(info)
    return info
